//
//
// Copyright 2020 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//

#ifndef GRPCPP_XDS_SERVER_BUILDER_H
#define GRPCPP_XDS_SERVER_BUILDER_H

#include <grpc/impl/codegen/port_platform.h>

#include <grpcpp/server_builder.h>

namespace grpc {
namespace experimental {

class XdsServerBuilder : public ::grpc::ServerBuilder {
 public:
  std::unique_ptr<Server> BuildAndStart() override {
    grpc_server_config_fetcher* fetcher =
        grpc_server_config_fetcher_xds_create();
    if (fetcher == nullptr) return nullptr;
    set_fetcher(fetcher);
    return ServerBuilder::BuildAndStart();
  }
};

}  // namespace experimental
}  // namespace grpc

#endif /* GRPCPP_XDS_SERVER_BUILDER_H */
